/////////////////////////////////////////////////////////////////
//Level loader script for Visual Level Editor
//Alexey Rozhkov 2007

var level_handle_n;//level handle
var level_loading;//determines if level is loaded or not

COLOR* amb_light = {red = 100;green = 90;blue = 70;}//ambient light color

ENTITY* level_sky;//selected in Visual Editor sky

//dummy function, used as default action in Visual Editor
void None()
{
	
}

/////////////////////////////////////////////////////////////////
//Functions for setting action and material

MATERIAL* mat_for_str(STRING* name)
{
	MATERIAL* temp_mat = *((MATERIAL**)engine_getvar(name,0));
	return(temp_mat);
}

void* func_for_str(char* name)
{
	void* temp_func = *((void**)engine_getvar(name,0));
	return(temp_func);
}

/////////////////////////////////////////////////////////////////
//Functions for loading objects

ENTITY* load_ent()
{	
STRING* error_str = str_create("#50");
STRING* read_file = str_create("#10");
STRING* read_material = str_create("#10");
STRING* read_action = str_create("#10");
ENTITY* read_ent;

if(file_str_read(level_handle_n,read_file)==-1){return(NULL);}//end of file? then exit
else
{
if(str_cmp(read_file,"[LIGHTS]")||str_cmp(read_file,"[CAMERA]")||str_cmp(read_file,"[SOUNDS]")){return(NULL);}//no more entities? then exit
}

//read file, material and action
file_str_read(level_handle_n,read_file);
file_str_read(level_handle_n,read_material);
file_str_read(level_handle_n,read_action);

void* temp_action = func_for_str(_chr(read_action));
MATERIAL* temp_mat = mat_for_str(_chr(read_material));

read_ent = ent_create(read_file,nullvector,temp_action);//create entity
read_ent.material = temp_mat;

//set material lighting
temp_mat.ambient_red = amb_light.red;
temp_mat.ambient_green = amb_light.green;
temp_mat.ambient_blue = amb_light.blue;

//read entity parameters
read_ent.x = file_var_read(level_handle_n);
read_ent.y = file_var_read(level_handle_n);
read_ent.z = file_var_read(level_handle_n);

read_ent.pan = file_var_read(level_handle_n);
read_ent.tilt = file_var_read(level_handle_n);
read_ent.roll = file_var_read(level_handle_n);

read_ent.scale_x = file_var_read(level_handle_n);
read_ent.scale_y = file_var_read(level_handle_n);
read_ent.scale_z = file_var_read(level_handle_n);

if(file_var_read(level_handle_n)){set(read_ent,FLAG1);}
else{reset(read_ent,FLAG1);}
if(file_var_read(level_handle_n)){set(read_ent,FLAG2);}
else{reset(read_ent,FLAG2);}
if(file_var_read(level_handle_n)){set(read_ent,FLAG3);}
else{reset(read_ent,FLAG3);}
if(file_var_read(level_handle_n)){set(read_ent,FLAG4);}
else{reset(read_ent,FLAG4);}
if(file_var_read(level_handle_n)){set(read_ent,FLAG5);}
else{reset(read_ent,FLAG5);}
if(file_var_read(level_handle_n)){set(read_ent,FLAG6);}
else{reset(read_ent,FLAG6);}
if(file_var_read(level_handle_n)){set(read_ent,FLAG7);}
else{reset(read_ent,FLAG7);}
if(file_var_read(level_handle_n)){set(read_ent,FLAG8);}
else{reset(read_ent,FLAG8);}

if(file_var_read(level_handle_n)){set(read_ent,PASSABLE);}
else{reset(read_ent,PASSABLE);}
if(file_var_read(level_handle_n)){set(read_ent,SHADOW);}
else{reset(read_ent,SHADOW);}
if(file_var_read(level_handle_n)){set(read_ent,INVISIBLE);}
else{reset(read_ent,INVISIBLE);}
if(file_var_read(level_handle_n)){set(read_ent,BRIGHT);}
else{reset(read_ent,BRIGHT);}
if(file_var_read(level_handle_n)){set(read_ent,OVERLAY);}
else{reset(read_ent,OVERLAY);}
if(file_var_read(level_handle_n)){set(read_ent,POLYGON);}
else{reset(read_ent,POLYGON);}
if(file_var_read(level_handle_n)){set(read_ent,CAST);}
else{reset(read_ent,CAST);}
if(file_var_read(level_handle_n)){set(read_ent,UNLIT);}
else{reset(read_ent,UNLIT);}
if(file_var_read(level_handle_n)){set(read_ent,TRANSLUCENT);}
else{reset(read_ent,TRANSLUCENT);}
if(file_var_read(level_handle_n)){set(read_ent,ZNEAR);}
else{reset(read_ent,ZNEAR);}

read_ent.alpha = file_var_read(level_handle_n);
read_ent.ambient = file_var_read(level_handle_n);
read_ent.albedo = file_var_read(level_handle_n);
read_ent.skill1 = file_var_read(level_handle_n);
read_ent.skill2 = file_var_read(level_handle_n);
read_ent.skill3 = file_var_read(level_handle_n);
read_ent.skill4 = file_var_read(level_handle_n);
read_ent.skill5 = file_var_read(level_handle_n);
read_ent.skill6 = file_var_read(level_handle_n);
read_ent.skill7 = file_var_read(level_handle_n);
read_ent.skill8 = file_var_read(level_handle_n);
read_ent.skill9 = file_var_read(level_handle_n);
read_ent.skill10 = file_var_read(level_handle_n);

return(read_ent);//return created entity
}

ENTITY* load_light()
{
ENTITY* read_light;
STRING* read_file = str_create("#10");

if(file_str_read(level_handle_n,read_file)==-1){return(NULL);}//end of file? then exit
else
{
	if(str_cmp(read_file,"[CAMERA]")||str_cmp(read_file,"[SOUNDS]")){return(NULL);}//no more lights? then exit
}

read_light = ent_create(NULL,nullvector,0);//create dummy entity
set(read_light,LIGHT|CAST);//it emits light

//read light parameters
read_light.lightrange = file_var_read(level_handle_n);
read_light.red  = file_var_read(level_handle_n);
read_light.green = file_var_read(level_handle_n);
read_light.blue = file_var_read(level_handle_n);
read_light.x = file_var_read(level_handle_n);
read_light.y = file_var_read(level_handle_n);
read_light.z = file_var_read(level_handle_n);

return(read_light);//return created light
}

ENTITY* load_sound()
{
ENTITY* read_sound;
SOUND* snd_file;
STRING* read_file = str_create("#10");

var read_volume;
var temp_handle;

if(file_str_read(level_handle_n,read_file)==-1){return(NULL);}//end of file? then exit
else{if(str_cmp(read_file,"[CAMERA]")){return(NULL);}}//no more sound sources? exit

read_sound = ent_create(NULL,nullvector,0);//create dummy entity

file_str_read(level_handle_n, read_file);
read_volume = file_var_read(level_handle_n);//read sound file name
snd_file = snd_create(read_file);//create sound at a runtime
temp_handle = ent_playloop(read_sound,snd_file,1000);//play it
snd_tune(temp_handle,read_volume,0,0);

//set position of sound source
read_sound.x = file_var_read(level_handle_n);
read_sound.y = file_var_read(level_handle_n);
read_sound.z = file_var_read(level_handle_n);

return(read_sound);//return created sound
}

void load_sky()
{
	STRING* read_str = "#10";
	file_str_read(level_handle_n, read_str);
	
   file_str_read(level_handle_n, read_str);
   if(level_sky){ent_remove(level_sky);}//remove previous sky
   level_sky = ent_createlayer(read_str, CUBE | SKY | VISIBLE, 1);//create new sky
   level_sky.flags2 &= ~INVISIBLE;//set it visible
}

void load_cam()//function for loading default camera position
{
	camera.x = file_var_read(level_handle_n);
	camera.y = file_var_read(level_handle_n);
	camera.z = file_var_read(level_handle_n);
	camera.pan = file_var_read(level_handle_n);
	camera.tilt = file_var_read(level_handle_n);
	camera.roll = file_var_read(level_handle_n);
	file_var_read(level_handle_n);
}

void load_fog()
{
	STRING* read_str = "#10";
	file_str_read(level_handle_n, read_str);
	
	//read and set fog properties
   d3d_fogcolor1.red = file_var_read(level_handle_n);
   d3d_fogcolor1.green = file_var_read(level_handle_n);
   d3d_fogcolor1.blue = file_var_read(level_handle_n);
   camera.fog_start = file_var_read(level_handle_n);
   camera.fog_end = file_var_read(level_handle_n);
   fog_color = file_var_read(level_handle_n);
}

void load_sun()
{
	STRING* read_str = "#10";
	file_str_read(level_handle_n, read_str);
	
	//read and set lighting properties
	sun_light = file_var_read(level_handle_n);
	sun_color.red = file_var_read(level_handle_n);
	sun_color.green = file_var_read(level_handle_n);
	sun_color.blue = file_var_read(level_handle_n);
	amb_light.red = file_var_read(level_handle_n);
	amb_light.green = file_var_read(level_handle_n);
	amb_light.blue = file_var_read(level_handle_n);
	sun_angle.pan = file_var_read(level_handle_n);
	sun_angle.tilt = file_var_read(level_handle_n);
	mat_shadow.alpha = file_var_read(level_handle_n);
}

/////////////////////////////////////////////////////////////////
//Functions for loading level

void new_level()
{
//set engine vars
d3d_entsort = 2;
terrain_chunk = 0;

//load dummy level
level_load("");
wait(2);

//create dummy sky
if(level_sky){ent_remove(level_sky);}
level_sky = ent_createlayer("skycube+6.tga", CUBE | SKY | VISIBLE, 1);
}

void load_level(STRING* name)
{
	var old_vol = sound_vol;//save volume
	//sound_vol = 0;//mute all sounds
	
	STRING* time_wasted = str_create("#150");
	float loading_time;
	level_loading = 1;
	timer();//start measuring loading time
	
	new_level();//empty level
	wait(3);
	level_handle_n = file_open_game(name);//open level file
	
	//load all objects
	load_sun();
	wait(1);
	load_sky();
	while(1){if(!load_ent()){break;}}
	while(1){if(!load_light()){break;}}
 	while(1){if(!load_sound()){break;}}
	load_cam();
	load_fog();
	
   file_close(level_handle_n);//close level file
   
   wait(5);
   level_loading = 0;//indicate that level is loaded(for use in other scripts)
   //sound_vol = old_vol;//set volume back
   
   //end_measuring loading time
   loading_time = timer()/1000000;
   str_for_num(time_wasted,loading_time);
   str_clip(time_wasted,1);
   str_cat(time_wasted," seconds");
   
   //enable this line to display level loading time
   //error(time_wasted);
}